from datetime import datetime, timedelta
import uuid
import unidecode
from random import choices, randint
import requests
import pytz
from .settings import Settings

INSCRIBE_CF_URL = 'https://us-central1-humai-cloud.cloudfunctions.net/cf_courses'
# PRIVILEGED_TOKEN = os.getenv('PRIVILEGED_TOKEN')

def timestamp():
    return datetime.now().strftime("%Y-%m-%d %H:%M:%S")

def generate_uuid():
    return str(uuid.uuid4())

def normalize(s):
    return unidecode.unidecode(s.lower().strip().replace(' ', '_'))

def generate_human_code():
    WORDS_URL = 'https://storage.googleapis.com/humai-datasets/nlp/borges_words.csv'
    words = requests.get(WORDS_URL).text.strip().split(',')
    return f"{choices(words)[0]}{''.join(str(randint(0,9)) for _ in range(3))}"


def inscribe_user(email: str, keyname: str, privileged_token: str = None, inscribe_cf_url: str = INSCRIBE_CF_URL) -> bool:
    """Inscripcion a un curso especifico utilizando `cf_courses` Cloud Function"""
    print(f"INFO inscribing {email} to {keyname}")
    data = {"action": "inscribe",
            'email': email,
            'keyname': keyname,
            'privileged_token': privileged_token}
    r = requests.post(inscribe_cf_url, json=data)
    print(f"inscribe cf response code = {r.status_code}")
    return r.ok


def inscribe_user_several_courses_by_id(email: str, courses_ids: list, privileged_token: str = None, inscribe_cf_url: str = INSCRIBE_CF_URL) -> bool:
    """Inscripcion a los cursos de una carrera utilizando `cf_courses` Cloud Function"""
    print(f"INFO inscribing {email} to courses ids {courses_ids}")
    data = {"action": "inscribe_several",
            'email': email,
            'courses_ids': courses_ids,
            'privileged_token': privileged_token}
    r = requests.post(inscribe_cf_url, json=data)
    print(f"inscribe cf response code = {r.status_code}")
    return r.ok


def get_member_inscriptions(member_id: int, directus_token: str, directus_base_url: str) -> list:
    """Obtener todos los registros de user_course del miembro member_id"""
    print(f"Getting inscriptions for member {member_id}")
    user_course_url = f"{directus_base_url}/items/user_course"
    headers = {"Authorization": f"Bearer {directus_token}"}
    params = {
        "filter[_and][0][member_id][_eq]": member_id,
        "filter[_and][1][course_id][_nnull]": "true",
        "fields[]": ["id", "course_id.id", "course_id.end_date", "course_id.keyname",  "course_id.name", "course_id.code", "course_id.discord_channel"]
    }
    get_usercourse_response = requests.get(user_course_url, headers=headers, params=params)
    user_course_info = get_usercourse_response.json().get('data', [])
    if len(user_course_info) == 0:
        print(f"User with member id  {member_id}, has not been inscribed in any course.")
        return []# None
    else:
        return user_course_info


def remove_member_inscription(member_id: int, user_course_info: list, settings: Settings) -> list:
    """"""
    print("removing course inscriptions")
    user_course_url = f"{settings.directus_base_url}/items/user_course"
    headers = {"Authorization": f"Bearer {settings.directus_access_token}"}
    # remove courses that hasn't finished neither started.
    user_course_ids_to_remove = []
    removed_course_keynames = []
    removed_course_info = []
    for user_course in user_course_info:
        print(f"user_course object {user_course}")
        if user_course["course_id"].get("end_date") is not None:
            end_date: datetime = datetime.fromisoformat(user_course["course_id"]["end_date"])
        else:
            end_date: datetime = datetime.now() + timedelta(days=1)
        print(f"course hasn't finished: {end_date > datetime.now()}")
        # Check for churn_date to not overwrite the past churn_date
        if end_date > datetime.now() and user_course.get('churn_date') is None:
            # the course hasn't finish
            user_course_ids_to_remove.append(user_course["id"])
            removed_course_keynames.append(user_course["course_id"]["keyname"])
            removed_course_info.append(user_course["course_id"])

    # update churn_date of courses
    if len(user_course_ids_to_remove) > 0:
        data = {
            "keys": user_course_ids_to_remove,
            "data": {
                "churn_date": datetime.now(pytz.timezone('America/Argentina/Buenos_Aires')).strftime('%Y-%m-%dT%H:%M:%S')
            }
        }
        update_response = requests.patch(user_course_url, headers=headers, json=data)

        if update_response.status_code == 200:
            print(f"Inscriptions for user with id {member_id} has been updated for courses {removed_course_keynames}")
        else:
            print(f"ERROR: Inscriptions could not be updated for user with id {member_id}, something has been wrong.")
    else:
        print(f"User with member id {member_id} has not active or incoming courses.")

    return removed_course_info


def send_discord_message(
        message, settings: Settings,
        channel_name: str = 'bugs-y-mejoras', thread: str = None
    ) -> bool:
    """
    Enviar un mensaje de notificacion al canal de discord
    :param message: mensaje con la informacion a enviar en el canal
    :param channel_name: nombre especifico del canal del servidor de discord
    """
    
    tito_headers = {
        "accept": "application/json",
        "access_token": settings.tito_api_key,
    }
    data = {
        'message': message,
        'channel_name': channel_name,
        'thread_name': thread
    }
    response = requests.post(f'{settings.tito_base_url}/send', params=data, headers=tito_headers)

    print(f'Se ejecuto envio el mensaje al canal {channel_name}: status code {response.status_code}')

    return response.ok



from .directus import *
from .mp import *
from .utils import *
from .settings import Settings
from .campus import *
from .permissions import *

__version__ = "0.0.3"
__author__ = 'Humai Dev Team'
__credits__ = 'Humai Dev Team'


from pydantic import BaseModel
import requests
from .settings import Settings


class DirectusSearch(BaseModel):
    table: str
    field: str
    operator: str
    value: str

def fetch_course_data(course_keyname, access_token, directus_base_url):
    """Refactor these functions, we could have only one search function"""
    course_search = DirectusSearch(
        **{"table": 'courses', "field": "keyname", "operator": "_eq", "value": course_keyname}
    )
    course_data = search_directus(course_search, access_token, directus_base_url)
    if course_data:
        open_courses = [c for c in course_data if c['inscription_open']]
        if len(open_courses) > 1:
            raise Exception(f'Multiple open courses found for {course_keyname}')
        else:
            course = open_courses[0]
            return course
    else:
        print(f'Course {course_keyname} not found')


def fetch_courses_by_id(courses_ids: list, directus_access_token: str, directus_base_url: str):
    """
    Al inscribirse a varios cursos contamos con una lista de ids
    pero no de sus keynames. Para esto buscamos obtener la info de los cursos
    """
    courses_search = DirectusSearch(**{"table": 'courses', "field": "id", "operator": "_in", "value": courses_ids})
    courses = search_directus(courses_search, directus_access_token, directus_base_url)

    return courses


def fetch_product_data(product_keyname, access_token, directus_base_url):
    product_search = DirectusSearch(
        **{"table": 'products', "field": "product_keyname", "operator": "_eq", "value": product_keyname}
    )
    product_data = search_directus(product_search, access_token, directus_base_url)
    if product_data:
        if len(product_data) > 1:
            raise Exception(f'Multiple products found for {product_keyname}')
        else:
            product = product_data[0]
            return product
    else:
        print('Course not found')


def search_directus(search: DirectusSearch, access_token, directus_base_url) -> list:
    """Applies search in table. This "directus search" is because I couldnt filter by passing a
    dictionary as the documentation specifies."""
    url = f"{directus_base_url}/items/{search.table}?filter[{search.field}][{search.operator}]={search.value}"
    headers = {"Authorization": f"Bearer {access_token}"}
    r = requests.get(url, headers=headers)
    data = r.json().get('data', [])
    if r.ok:
        if (n := len(data)) == 1:
            return data
        elif n > 1:
            print('Multiple results found')
            return data
        else:
            print('No Directus results found')
            return data
    else:
        raise Exception(f'Error in Directus search {r.url}')


def fetch_all(table: str, access_token: str, directus_base_url: str, limit: int = -1) -> list:
    """
    Applies search in table. This "directus search" is because I couldnt filter by passing a
    dictionary as the documentation specifies.

    Args:
        table: name of directus table to fetch
        access_token: directus token
        directus_base_url: directus app url
        limit: amount of register fetched (default is -1, means unlimited)

    Returns: a list of the data where each item is a row.

    """
    url = f"{directus_base_url}/items/{table}"
    headers = {"Authorization": f"Bearer {access_token}"}
    r = requests.get(url, headers=headers, params={"limit": limit})
    if r.ok:
        return r.json()['data']
    else:
        raise Exception(f'Error in Directus search {r.url}, {r.content}, {r.status_code}')


def post_directus(table: str, data: dict, access_token: str, directus_base_url: str) -> requests.Response:
    """Post data to table"""
    url = f"{directus_base_url}/items/{table}"
    headers = {"Authorization": f"Bearer {access_token}"}
    r = requests.post(url, json=data, headers=headers)
    return r


def patch_directus(table: str, data: dict, item_id: str, access_token: str, directus_base_url: str) -> requests.Response:
    """Update item in table"""
    url = f"{directus_base_url}/items/{table}/{item_id}"
    headers = {"Authorization": f"Bearer {access_token}"}
    r = requests.patch(url, json=data, headers=headers)
    return r

def get_coupon_usage_count(directus_base_url, directus_token, coupon):
    directus_headers = {'Authorization': f'Bearer {directus_token}'}
    get_pre_insc_url = f"{directus_base_url}/items/pre_inscription_info/?filter[coupon_code][_eq]={coupon}"
    pre_ins_info_response = requests.get(get_pre_insc_url, headers=directus_headers)

    mp_ext_ref = set()
    pp_ext_ref = set()
    uses = 0
    for pre_insc in pre_ins_info_response.json()['data']:
        if pre_insc.get('payment_platform') == 'mercadopago':
            mp_ext_ref.add(pre_insc.get('external_reference'))
        if pre_insc.get('payment_platform') == 'paypal':
            pp_ext_ref.add(pre_insc.get('external_reference'))
    
    if len(mp_ext_ref) > 0:
        params = {
            'filter[external_reference][_in]': list(mp_ext_ref)
        }
        mp_response = requests.get(f"{directus_base_url}/items/Payments", headers=directus_headers, params=params)
        if mp_response.ok:
            uses += len(set((payment.get('external_reference') for payment in mp_response.json()['data'])))

    if len(pp_ext_ref) > 0:
        params = {
            'filter[external_reference][_in]': list(pp_ext_ref)
        }
        pp_response = requests.get(f"{directus_base_url}/items/pp_payments", headers=directus_headers, params=params)
        if pp_response.ok:
            uses += len(set((payment.get('external_reference') for payment in pp_response.json()['data'])))

    # uses = len(pre_ins_info_response.json()['data'])
    return uses

def fetch_coupon_discount(coupon, access_token, directus_base_url) -> int:
    MEMBER_CODE_DISCOUNT = 20
    if not coupon:
        return 0

    
    # Search in coupon table
    coupon_search = DirectusSearch(
        **{"table": 'coupons', "field": "coupon", "operator": "_eq", "value": coupon}
    )
    coupon_data = search_directus(coupon_search, access_token, directus_base_url)
    if coupon_data:
        if not (n := coupon_data[0]['max_uses']):
            return coupon_data[0]['discount']
        else:
            coupon_usage_count = get_coupon_usage_count(directus_base_url, access_token, coupon)
            if  n > coupon_usage_count:
                return coupon_data[0]['discount']
            else:
                print(f'Coupon {coupon} no longer available.')
            
    # Search in member table
    member_coupon_search = DirectusSearch(
        **{"table": 'members', "field": "member_code", "operator": "_eq", "value": coupon}
    )
    member_coupon_data = search_directus(member_coupon_search, access_token, directus_base_url)
    if member_coupon_data:
        return MEMBER_CODE_DISCOUNT
    
    # No result
    return 0

def get_mail_info_directus(email_name: str, access_token, directus_base_url):
    '''Obtener el mail de bienvenida desde directus'''
    # deberia estar en humai tools

    email_search_body = {
        "table": "emails",
        "field": "name",
        "operator": "_eq",
        "value": email_name
    }  # complete value to send mail when is unique course payment
    email_info_search = DirectusSearch(**email_search_body)
    email_info_response = search_directus(search=email_info_search,
                                            directus_base_url=directus_base_url,
                                            access_token=access_token)

    return email_info_response

def insert_user_event(
    member_id: str,
    event_type: str,
    settings: Settings,
    reason: str = "",
    product_credits: int = 0,
) -> bool:
    """"""
    event_to_insert = {
        "category": "credits",
        "member_id": int(member_id),
        "event_type": event_type,
        "amount": product_credits,
        "description": reason
    }

    post_response = post_directus(
        table="user_events",
        data=event_to_insert,
        access_token=settings.directus_access_token,
        directus_base_url=settings.directus_base_url,
    )

    if post_response.status_code == 200:
        print(f"member {member_id} user_event has been inserted.")
        return True
    else:
        print(f"member {member_id} user_event has not been inserted.")
        return False


from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file_encoding='utf-8', env_ignore_empty=True)

    directus_access_token: str
    directus_base_url: str = Field(default='https://directus-dev-mrek6efuua-uc.a.run.app')

    tito_api_key: str
    tito_base_url: str = Field(default='http://tito.humai.com.ar')

    campus_base_url: str = Field(default='https://estudiantes.campus.humai.com.ar')
    campus_client_id: str = Field(pattern=r'\w+')
    campus_client_secret: str = Field(pattern=r'\w+')
    campus_access_token: str = Field(default=None, validate_default=False)



import requests
from .directus import fetch_courses_by_id, fetch_course_data, patch_directus, search_directus, DirectusSearch
from . import campus
from .settings import Settings
from datetime import datetime
import pytz


def post_discord_permission(
        new_courses: list, member_mail: str, action_type: str,
        settings: Settings
    ) -> bool:
    """
    Request a tito para actualizar los accesos de los usuarios en discord
    """
    headers = {
        "accept": "application/json",
        "access_token": settings.tito_api_key,
    }

    discord_search = DirectusSearch(table='discord_info', field='mail', operator='_eq', value=member_mail)
    discord_response = search_directus(discord_search, settings.directus_access_token, settings.directus_base_url)
    if len(discord_response) == 1:
        discord_data = discord_response[0]
    else:
        print(f"Member {member_mail} not in discord_info")
        return False


    discord_data_post = {
        'user': discord_data['username']
    }
    if action_type == 'add':
        discord_access = set([access
            for access in discord_data.get('access', '').strip().split(',')
            if len(access) > 0])
        discord_data_post['access'] = ','.join(discord_access.union(set(new_courses)))

    elif action_type == 'remove':
        discord_data_post['access'] = 'restricted'
        
    discord_response = requests.post(f'{settings.tito_base_url}/set_permissions', params=discord_data_post, headers=headers)

    return discord_response.status_code == 200


def discord_add_role(role_name: str, settings: Settings, email: str = None, username: str = None):
    """
    Add a role to a user in career inscription
    """
    headers = {
        "accept": "application/json",
        "access_token": settings.tito_api_key,
    }
    discord_data_post = {
        'role_name': role_name,
        'email': email,
        'user': username
    }
    discord_response = requests.post(f'{settings.tito_base_url}/roles', params=discord_data_post, headers=headers)

    return discord_response.status_code == 200


def deactivate_membership(member_id: int, directus_token: str, directus_base_url: str) -> requests.Request:
    """"""
    print(f"Deactivating membership for {member_id}")
    table = 'members'
    churn_date = datetime.now(pytz.timezone('America/Argentina/Buenos_Aires')).strftime('%Y-%m-%dT%H:%M:%S')
    data = {'is_active': False, 'is_suscriptor': False, 'churn_date': churn_date}
    update_response = patch_directus(table, data, member_id, directus_token, directus_base_url)
    return update_response


def deactivate_contribution(member_id: int, directus_token: str, directus_base_url: str) -> requests.Request:
    """"""
    print(f"Deactivating contribution for {member_id}")
    table = 'members'
    churn_date = datetime.now(pytz.timezone('America/Argentina/Buenos_Aires')).strftime('%Y-%m-%dT%H:%M:%S')
    data = {'is_contributor': False, 'churn_date': churn_date}

    update_response = patch_directus(table, data, member_id, directus_token, directus_base_url)
    return update_response


def process_enroll_course(member_mail: str, course_keyname: str, settings: Settings):
    """
    Dar de alta al curso correspondiente al keyname
    """
    course_info = fetch_course_data(course_keyname, settings.directus_access_token, settings.directus_base_url)
    if settings.campus_access_token is None:
        campus_token = campus.get_campus_jwt_token(settings)
        settings.campus_access_token = campus_token

    enroll_status = campus.handle_campus_inscription(member_mail, course_info.get('code'), settings)
    if not enroll_status:
        print(f'Error en campus: {course_keyname} del usuario {member_mail}')


    discord_response = post_discord_permission([course_keyname], member_mail, action_type='add', settings=settings)
    if not discord_response:
        print(f'Error al asignar acceso a {course_keyname} en discord al usuario {member_mail}')


def process_enroll_career(member_mail: str, courses_ids: str, career_role: str, settings: Settings):
    """
    Dar de alta a los cursos correspondiente a la carrera
    """
    courses = fetch_courses_by_id(courses_ids, settings.directus_access_token, settings.directus_base_url)
    settings.campus_access_token = campus.get_campus_jwt_token(settings)
    courses_codes = [c["code"] for c in courses]

    campus_response = campus.handle_campus_inscription(member_mail, courses_codes, settings=settings)
    if not campus_response:
        # TBD: manejar error a la inscripcion del campus
        print('Error en la inscripcion')

    # Inscribe in discord
    discord_response = post_discord_permission([career_role], action_type='add', settings=settings, member_mail=member_mail)
    if not discord_response:
        print(f'Error al asignar el rol {career_role} en discord al usuario {member_mail}')


def process_cancel_membership(
        member_mail: str,
        settings: Settings,
        member_id: int
    ):
    """
    Funcion para gestionar la baja de un miembro
    """

    campus_token = campus.get_campus_jwt_token(settings.campus_client_id, settings.campus_client_secret)
    username = campus.get_campus_username(member_mail, campus_token)

    unenroll: bool = campus.unroll_campus_course(username, campus_token)
    # unenroll: bool = campus.unroll_all_campus_courses(username, campus_token)

    discord_response = post_discord_permission(
        ['restricted'], member_mail,
        action_type='remove', 
        settings=settings
    )
    if not discord_response:
        print(f'Error al remover acceso en discord al usuario {member_mail}')

    member_response = deactivate_membership(member_id, settings.directus_access_token, directus_base_url=settings.directus_base_url)
    if not member_response.ok:
        print(f'Error al desactivar al usuario {member_mail} de directus')

    return unenroll


import requests
import re
import base64
from .utils import generate_human_code
from .settings import Settings


def get_edx_id(course: dict) -> str:
    """Obtener el course_id a partir de la info en directus"""
    # course-v1:Humai+python+python17.8.23
    # return course[EDX_COURSE_ID_KEY]
    return f'course-v1:Humai+{course["keyname"]}+{course["code"]}'

def get_edx_id_from_code(course_code: str) -> str:
    """Obtener el course_id a partir del code del curso"""
    # course-v1:Humai+python+python17.8.23
    keyname = re.search(r"^\D+", course_code.replace(" ", "").lower())[0]
    return f'course-v1:Humai+{keyname}+{course_code}'


def get_campus_jwt_token(settings: Settings) -> str:
    """
    Obtiene el Auth token para poder realizar acciones de staff
    con la api del campus
    """
    auth_token = base64.b64encode(f'{settings.campus_client_id}:{settings.campus_client_secret}'.encode("utf-8")).decode("utf-8")
    headers = {
        "Authorization": f"Basic {auth_token}", 
        "Cache-Control": "no-cache"
    }
    data = {
        "grant_type": "client_credentials", 
        "token_type": "jwt"
    }

    r = requests.post(
        f"{settings.campus_base_url}/oauth2/access_token", headers=headers, data=data
    )

    if r.ok:
        settings.campus_access_token = r.json()["access_token"]
        return r.json()["access_token"]
    else:
        raise ValueError(f'Se genero un error al obtener el access_token (status {r.status_code})')
    

def create_campus_user(member_mail: str, username: str, name:str, settings: Settings,
                       password: str = None, course_id: str = None) -> str:
    """
    Crear un usuario en el campus de humai
    Args:
        password (optional): se genera un password nuevo si no se adjunta
    """
    if password is None:
        password = generate_human_code()

    print(f'Creando usuario con mail {member_mail}')
    headers = {'Authorization': f'JWT {settings.campus_access_token}'}
    create_user_data = {
        "email": member_mail,
        "name": name,
        "username": username,
        "password": password,
        "terms_of_service": "true",
    }

    # Enroll mientras se crea el usuario
    if course_id is not None:
        create_user_data['course_id'] = course_id

    # Se debe guardar la informacion de registro
    response_create = requests.post(f'{settings.campus_base_url}/api/user/v1/account/registration/', 
                                    data=create_user_data, headers=headers)

    if response_create.status_code == 200:
        print(f'El usuario {create_user_data["username"]} se creo con exito. password: {password}')

        return create_user_data["username"]

    elif response_create.status_code == 209:
        try:
            error_info = response_create.json()
            print(f'El error obtenido es {error_info.get("error_code")}')
        except Exception:
            pass
        print(f'El usuario con mail {member_mail} ya existe')

    else:
        print(f'Error {response_create.status_code} al crear al usuario con mail {member_mail}')


def get_campus_username(member_mail: str, settings: Settings) -> str:
    """
    Obtener el username de edX para poder realizar el resto de requests
    """
    params = {'email': member_mail}
    headers = {'Authorization': f'JWT {settings.campus_access_token}'}
    response_user = requests.get(f'{settings.campus_base_url}/api/user/v1/accounts', params=params, headers=headers)

    if response_user.status_code == 200:
        users: list = response_user.json()
        return users[0]['username']
    
    elif response_user.status_code == 401:
        print('Error: Not Athorized')

    elif response_user.status_code == 404:
        # No user exists with the specified mail
        print(f'Error: No existe usuario con mail {member_mail}')

    else:
        print(f'ERROR {response_user.status_code}: user_email {member_mail}')


def get_users_info(usernames: list, settings: Settings):
    headers={"Authorization": f"JWT {settings.campus_access_token}"}

    params = {
        'username': ",".join(usernames)
    }
    r = requests.get(f'{settings.campus_base_url}/api/user/v1/accounts', params=params, headers=headers)

    if r.ok:
        print('Info de los usuarios')
        return r.json()
    else:
        raise ValueError(f'Error: no se pudo obtener la informacion de {len(usernames)} usuarios')


def replace_campus_username(old_username: str, new_username: str, settings: Settings) -> bool:
    """
    Actualizar el username de una persona por uno nuevo
    """
    headers = {'Authorization': f'JWT {settings.campus_access_token}'}
    post_data = {
        'username_mappings': [
            {old_username: new_username}
        ]
    }
    response = requests.get(f'{settings.campus_base_url}/api/user/v1/accounts/replace_usernames/', json=post_data, headers=headers)

    if response.status_code == 200:
        usernames = response.json()
        print(usernames.get('successful_replacements'))
        print(f'Se actualizo el usuario {old_username} con el nuevo username {new_username}')
        return True
    
    else:
        print(f'ERROR {response.status_code}: user_email {old_username}. {response.text}')
        return False


def handle_campus_inscription(
        member_mail: str, courses_code: str, settings: Settings,
        action: str = "enroll", auto_enroll: bool = True, email_students: bool = False,
    ) -> bool:
    """
    Args:
        member_mail: member mail used to send enroll notification
        course_id: id generated in OpenEdx
        auto_enroll (bool): inscribe al usuario de forma automatica una vez registrado
        email_students (bool): el usuario recibe un email con la notificacion de la accion
    Info:
        action: determina el comportamiento segun los valores "enroll" o "unenroll"
    Returns:
        bool: the status of the request
    """

    if action not in ["enroll", "unenroll"]:
        print(f"Error invalid action argument: {action}")
        return False

    courses_ids = [get_edx_id_from_code(course_code) 
                   for course_code in courses_code.split(",")]
    headers = {
        'Authorization': f'JWT {settings.campus_access_token}'
    }
    data = {
        'action': action,
        'identifiers': member_mail,
        'auto_enroll': auto_enroll,
        'email_students': email_students,
        'courses': ','.join(courses_ids)
    }

    r = requests.post(f'{settings.campus_base_url}/api/bulk_enroll/v1/bulk_enroll', headers=headers, json=data)

    if r.status_code == 200:
        print(f'CAMPUS: El usuario {member_mail} se inscribio correcamente a {courses_code}')
    else:
        print(f'ERROR CAMPUS: no se inscribio al usuario {member_mail} a {courses_code}')

    return r.status_code == 200


def get_enrollments(username: str = None, course_id: str = None, settings: Settings = None):
    """
    Return all the inscriptions from a user or course.
    token: pass as argument `access_token` or assign `CAMPUS_ACCESS_TOKEN` env variable
    """

    headers={"Authorization": f"JWT {settings.campus_access_token}"}
    data = list()

    params = {
        'page_size': 300
    }
    if username is not None:
        params['username'] = username

    if course_id is not None:
        params['course_id'] = course_id

    r = requests.get(f'{settings.campus_base_url}/api/enrollment/v1/enrollments', params=params, headers=headers)

    if r.ok:
        print(f'Inscripciones status: {r.status_code}, next: {r.json()["next"]}')
        data.extend(r.json()['results'])
        while r.json()["next"]:
            r = requests.get(r.json()["next"], params=params, headers=headers)
            if r.ok:
                data.extend(r.json()['results'])
        return data
    else:
        raise ValueError('Error: no se pudo obtener la informacion de inscripciones.')


def enroll_campus_course(username: str, course_id: str, settings: Settings) -> bool:
    """Dar de alta en el curso especificado"""

    headers = {'Authorization': f'JWT {settings.campus_access_token}'}
    enroll_data = {
        'user': username,
        'is_active': True,
        'course_details': {'course_id': course_id},
    }
    enroll_response = requests.post(f'{settings.campus_base_url}/api/enrollment/v1/enrollment', headers=headers, json=enroll_data)

    if enroll_response.status_code == 200:
        print(f'The user {username} has been successfully enrolled in {course_id}')
        enrollments: dict = enroll_response.json()
        user_courses = enrollments['course_details']['course_name']
        print(f'User enrolled to {user_courses}')
        return True
    
    else:
        print(f'ERROR {enroll_response.status_code} with enroll user {username} in course {course_id}')
        return False


def unroll_campus_course(username: str, settings: Settings, course_id: str = None) -> bool:
    """
    Dar de baja a un usuario a un curso unico o todos en edX
        course_id != None: dar de baja al curso especifico (Bearer token)
        course_id == None: dar de baja a todos los cursos  (JWT token)
    """
    unrollments_names = list()
    headers = {'Authorization': f'JWT {settings.campus_access_token}'}

    if course_id is not None:
        courses_ids = course_id.split(',')
    else: 
        courses_ids = [c.get('course_id') for c in get_enrollments(username=username, settings=settings) if c.get('is_active')]

    for id in courses_ids:
        unenroll_data = {
            'user': username,
            'is_active': False,
            'course_details': {'course_id': id},
        }
        response = requests.post(f'{settings.campus_base_url}/api/enrollment/v1/enrollment', headers=headers, json=unenroll_data)

        if response.status_code == 200:
            course_name = response.json().get('course_details', {}).get('course_name')
            unrollments_names.append(course_name)
        else:
            print(f'ERROR {response.status_code}: with user {username} in course {id}')

    # Para debug
    if len(unrollments_names) > 0:
        print(f'El usuario {username} se dio de baja a {unrollments_names}')
    
    return len(unrollments_names) > 0


def unroll_all_campus_courses(username: str, settings: Settings) -> bool:
    """
    Unenroll username to all courses
    https://github.com/openedx/edx-platform/blob/master/openedx/core/djangoapps/enrollments/views.py#L389

    Unused because returns: 404 Error with text 'No retirement request status for username.'
    """
    headers = {'Authorization': f'JWT {settings.campus_access_token}'}
    unenroll_data = {
        'username': username
    }
    response = requests.post(f'{settings.campus_base_url}/api/enrollment/v1/unenroll', headers=headers, json=unenroll_data)


    if response.status_code == 200:
        unenrollments: list = response.json()
        # user_courses = unenrollments['course_details']['course_name']
        print(f'User unenrolled from {unenrollments}')
        return True
    
    if response.status_code == 204:
        print('The user is already unenrolled from all courses')
        return True
    
    else:
        print(f'ERROR {response.status_code}: with user {username} in unroll all')
        return False
    

def get_campus_courses_info(settings: Settings) -> list:
    """Obtener informacion basica de los cursos del campus"""
    headers={"Authorization": f"JWT {settings.campus_access_token}"}
    course_info = list()
    r = requests.get(f'{settings.campus_base_url}/api/courses/v1/courses/', headers=headers)

    if r.ok:
        course_info.extend(r.json()['results'])
        while (new_url := r.json()["pagination"].get("next")):
            r = requests.get(new_url, headers=headers)
            if r.ok:
                course_info.extend(r.json()['results'])
        print(f'Informacion cursos, next: {r.json()["pagination"].get("next")}')
        return course_info
    else:
        raise ValueError(f'Error: {r.text}.')


def get_problems_content(code: str, settings: Settings, keyname: str = None):
    
    headers={"Authorization": f"JWT {settings.campus_access_token}"}
    responses = list()

    if keyname is None or len(keyname) == 0:
        keyname = re.search(r'^\D+', code)[0]

    r = requests.get(f'{settings.campus_base_url}/api/grades/v1/submission_history/course-v1%3AHumai%2B{keyname}%2B{code}/', headers=headers)

    if not r.ok:
        print('Se genero un erro en la primera iteracion')
        print(r.text)
        return []

    if len(r.json()['results']) == 0:
        raise ValueError(f"No se pudo obtener datos del curso {keyname}+{code}.\
            Revisar el keyname o code asignado en el campus"
        )

    responses += r.json()['results']
    while r.json().get('next') is not None:
        r = requests.get(r.json()['next'], headers=headers)

        if r.ok:
            responses += r.json()['results']

    return responses


def get_xblocks(code: str, settings: Settings):
    headers={"Authorization": f"JWT {settings.campus_access_token}"}

    params = {
        'username': 'admin',
        'depth': 'all',
        'block_types_filter': 'problem',
        'student_view_data': 'problem',
        'return_type': 'list',
        'course_id': get_edx_id_from_code(code)
    }
    url = f'{settings.campus_base_url}/api/courses/v2/blocks/'
    r = requests.get(url, params=params, headers=headers)

    if r.ok:
        print('Info de problemas')
        return r.json()
    else:
        raise ValueError(f'Error: {r.status_code} - {r.text}')